% periodicSolution.m
%
% Computes the periodic solution \tilde{P} with \kappa=50 of Sec. IV-B.
%
% Method:
% Find a \kappa-periodic solution to the iteration of reduced communication 
% Kalman filter (eq. (21)), that is, find a matrix P that satisfies
%
%   P = G_cal^{50}(P)
%
% This is done by fixed point iteration of the equation
%
%   P(k+1) = G_cal^{50}(P(k))
%
% and observing the convergence of the solutions.
% The obtained result is the matrix \tilde{P} in Sec. IV-B.
%
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Reduced Communication State Estimation for 
% Control of an Unstable Networked Control System", 50th IEEE Conference 
% on Decision and Control and European Control Conference, 2011.
%
% Copyright:
% Sebastian Trimpe
% ETH Zurich
% Institute for Dynamic Systems and Control (IDSC)
% strimpe(at)ethz.ch
% September 2011
%


%% Load Kalman filter parameters.
% See corresponding script for details.
KalmanFilterParameters;



%% Parameters for fix-point iteration.
% Period of the solution (can be observed from the simulation results
% obtained with simulate.m).
kappa = 50;

% Maximum number of iterations
max_iter = 200;

% Tolerance: Iteration terminates if the matrix-2-norm of the change in P  
% is less than this tolerance.
tol = 1e-9;



%% Fix-point iteration.
% Iteration of the Kalman filter variance.  We seek a kappa-periodic
% solution, that is, we'll seek a fixed point of 
%
%       P = G_cal^{kappa}(P).
%

% Initialize with the solution of the DARE.
P = P_bar;

% Variables used in iteration.
P_prev = zeros(n,n);        % previous step
iter = 0;

% Keep track of sending decisions over one period
gamma = nan(kappa,size(C,1));

% Iterate until numeric solution has converged (the norm of update step is
% less than tolerance) or maximum number of iterations is reached.
while norm(P-P_prev)>tol && iter<max_iter
    P_prev = P;
    
    % Apply G_cal (eq. (34), (35)) kappa times.
    for k=1:kappa
        [P,gamma(k,:)] = G_cal(P,A,C,Q,R,P_bar,delta);
    end;
    
    % Increment iteration variable
    iter = iter+1;
    
    % Display:
    disp(['Iter = ',num2str(iter,'%d'),'    ||P-P_prev|| = ',num2str(norm(P-P_prev),'%10.5e'),'    ||P|| = ',num2str(norm(P),'%10.5e'),'.']);
end;



%% Display iteration results.
disp(' ');
if norm(P-P_prev)<=tol
    disp('Fix-point iteration converged. (Change in variance less than specified threshold.)');
elseif iter==max_iter
    disp('Fix-point iteration may not have converged. (Maximum number of iterations reached.)');
else
    disp('Unknown termination.');
end;

% result
disp(' ');
disp('Result: P = ');
disp(P);

% Result with small values set to zero for better readability.
% Set entries to zero that are in [-0.005,0.005].
clippingValue = 0.005;
for r = 1:size(P,1)
    for c = 1:size(P,2)
        if abs(P(r,c)) < clippingValue
            P(r,c) = 0;
        end;
    end;
end;
%
disp(' ');
disp('Result (values rounded to 0.01): P = ');
disp(round(P*100)/100);     % round to second decimal place

disp(' ');
disp('This is the result for $\tilde{P}$ in Sec. IV-B. of the paper ');
disp('"Reduced Communication State Estimation for Control of an Unstable Networked Control System".');
disp(' ');

    